@@ -1,6 +1,6 @@
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://app/modules/urlHelper.jsm");
+Components.utils.import("resource://gre/modules/urlHelper.jsm");
 
 const nsISupports            = Components.interfaces.nsISupports;
 
@@ -123,13 +123,13 @@ nsBlueGriffonContentHandler.prototype = {
       return;
 
     var url = null;
-//@line 127 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 127 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
     if (cmdLine.length == 1) {
       var arg = cmdLine.getArgument(0);
       if (arg[0] != "-")
         url = arg;
     }
-//@line 133 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 133 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
 
     var ar = cmdLine.handleFlagWithParam("file", false);
     if (ar || url) {
@@ -215,13 +215,13 @@ nsDefaultCommandLineHandler.prototype = {
   handle : function dch_handle(cmdLine) {
 
     var url = null;
-//@line 219 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 219 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
     if (cmdLine.length == 1) {
       var arg = cmdLine.getArgument(0);
       if (arg[0] != "-")
         url = arg;
     }
-//@line 225 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 225 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
 
     var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                        .getService(nsIWindowMediator);
@@ -252,12 +252,12 @@ nsDefaultCommandLineHandler.prototype = {
         return;
       }
 
-//@line 258 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 258 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
       return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
                                "_blank",
                                "chrome,dialog=no,all",
                                ar || url);
-//@line 263 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 263 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
     }
     url = cmdLine.handleFlagWithParam("url", false);
     if (url) {
@@ -280,17 +280,17 @@ nsDefaultCommandLineHandler.prototype = {
       mostRecent.focus();
       return mostRecent;
     }
-//@line 293 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 293 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
     if (!cmdLine.preventDefault)
       return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
                                "_blank",
                                "chrome,dialog=no,all",
                                "");
-//@line 299 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 299 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
 
   },
 
-//@line 317 "/build/bluegriffon/src/mozilla-central-fec90cbfbaad/bluegriffon/components/bgCommandHandler.js"
+//@line 317 "/build/bluegriffon/src/mozilla-central-f14898695ee0/bluegriffon/components/bgCommandHandler.js"
 
   helpInfo : ""
 };
